"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BitcoinCoreClient = void 0;
const bjs = __importStar(require("bitcoinjs-lib"));
const bip32 = __importStar(require("bip32"));
const networks_1 = require("../utils/networks");
const rpc_client_1 = require("../utils/rpc-client");
class BitcoinCoreClient {
    constructor(url, username, password, network) {
        this.rpc = new rpc_client_1.RpcClient(url, username, password);
        this.network = networks_1.Networks[network];
    }
    async _getTransaction(hash) {
        return this.rpc.call("gettransaction", hash);
    }
    async getAddressTransactions(address) {
        const receivedByAddress = await this.rpc.call("listreceivedbyaddress", 0, false, true, address);
        if (receivedByAddress.length === 0)
            return [];
        const txs = await Promise.all(receivedByAddress[0].txids.map((hash) => this._getTransaction(hash)));
        return txs.map((tx) => {
            const transaction = {
                hash: tx.txid,
                confirmations: tx.confirmations,
                amount: Math.abs(tx.amount),
            };
            return transaction;
        });
    }
    async importWallet(xpub, numAddresses) {
        const baseNode = bip32.fromBase58(xpub, this.network);
        const addresses = [];
        for (let i = 0; i < numAddresses; i++) {
            const publicKey = baseNode.derive(0).derive(i).publicKey;
            const address = bjs.payments.p2wpkh({
                pubkey: publicKey,
                network: this.network,
            }).address;
            addresses.push(address);
        }
        const addrRequest = addresses.map((addr) => ({
            scriptPubKey: { address: addr },
            watchonly: true,
            timestamp: "now",
        }));
        return this.rpc.call("importmulti", addrRequest);
    }
}
exports.BitcoinCoreClient = BitcoinCoreClient;
