"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EsploraClient = void 0;
const axios_1 = __importDefault(require("axios"));
class EsploraClient {
    constructor(url) {
        this.url = url;
    }
    async _getBlockHeight() {
        // cache it
        return (await axios_1.default.get(`${this.url}/blocks/tip/height`)).data;
    }
    async getAddressTransactions(address) {
        const height = await this._getBlockHeight();
        const txs = (await axios_1.default.get(`${this.url}/address/${address}/txs`)).data;
        return txs.map((tx) => {
            const confirmations = tx.status.confirmed
                ? height - tx.status.block_height
                : 0;
            const output = tx.vout.find((vout) => vout.scriptpubkey_address === address);
            const transaction = {
                hash: tx.txid,
                amount: output.value / 1e8,
                confirmations,
            };
            return transaction;
        });
    }
}
exports.EsploraClient = EsploraClient;
